﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;

namespace MvcConf2011.Areas.CustomValidation.Controllers
{
    public class MyNumericModel
    {
        [Display(Name = "The First One")]
        public int Value1 { get; set; }

        [GreaterThan("Value1")]
        [Display(Name = "The Second One")]
        public int Value2 { get; set; }
    }

    public class CustomValidationController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        [HttpPost]
        public ActionResult Index(MyNumericModel model)
        {
            return View(model);
        }
    }

    public class GreaterThanAttribute : ValidationAttribute, IClientValidatable
    {
        public GreaterThanAttribute(string otherProperty)
        {
            ErrorMessage = "{0} must be greater than {1}.";
            OtherProperty = otherProperty;
        }

        public string OtherProperty { get; private set; }

        string FormatErrorMessage(string thisPropertyDisplayName, string otherPropertyDisplayName)
        {
            return String.Format(ErrorMessageString, thisPropertyDisplayName, otherPropertyDisplayName);
        }

        public IEnumerable<ModelClientValidationRule> GetClientValidationRules(ModelMetadata metadata, ControllerContext context)
        {
            var otherPropertyMetadata = ModelMetadataProviders.Current.GetMetadataForProperty(null, metadata.ContainerType, OtherProperty);

            var rule = new ModelClientValidationRule
            {
                ErrorMessage = FormatErrorMessage(
                    metadata.GetDisplayName(),
                    otherPropertyMetadata.GetDisplayName()
                ),
                ValidationType = "greater",
            };

            rule.ValidationParameters["other"] = "*." + OtherProperty;

            yield return rule;
        }

        protected override ValidationResult IsValid(object value, ValidationContext validationContext)
        {
            var otherProperty = validationContext.ObjectType.GetProperty(this.OtherProperty);
            var otherValue = (int)otherProperty.GetValue(validationContext.ObjectInstance, null);
            var thisValue = (int)value;

            if (thisValue > otherValue)
                return null;

            var otherPropertyMetadata = ModelMetadataProviders.Current.GetMetadataForProperty(() => validationContext.ObjectInstance, validationContext.ObjectType, OtherProperty);

            return new ValidationResult(
                FormatErrorMessage(
                    validationContext.DisplayName,
                    otherPropertyMetadata.GetDisplayName()
                )
            );
        }
    }
}
